unit Main_F;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls,
  SDUForms, ExtCtrls, SDUFrames, SDUFilenameEdit_U;

type
  TMain = class(TSDUForm)
    pbCheck: TButton;
    Label1: TLabel;
    reReport: TRichEdit;
    Label2: TLabel;
    Label3: TLabel;
    OpenDialog1: TOpenDialog;
    pbClose: TButton;
    rgParamCheck: TRadioGroup;
    feOriginal: TSDUFilenameEdit;
    feTranslated: TSDUFilenameEdit;
    procedure pbCheckClick(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    function  CheckParameters(original: string; translation: string): boolean;
    function  StripToPlaceholderes(s: string): string;
    function  CheckParameters_123Style(strippedOriginal: string; strippedTranslation: string): boolean;

    function  LoadTranslation(original: TStringList; translation: TStringList): boolean;
    procedure JoinLines(input: TStringList);
  public
  end;

var
  Main: TMain;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral;

// Join lines which span multiple lines
procedure TMain.JoinLines(input: TStringList);
var
  i: integer;
begin
  for i:=(input.count - 1) downto 0 do
    begin
    if (length(input[i]) > 0) then
      begin
      if (input[i][1] = '"') then
        begin
        input[i-1] := Copy(input[i-1], 1, (length(input[i-1])-1)) +
                      Copy(input[i], 2, (length(input[i])-1));
        input.Delete(i);
        end;
      end;

    end;
end;

function TMain.LoadTranslation(original: TStringList; translation: TStringList): boolean;
var
  allOK: boolean;
begin
  allOK := TRUE;

  original.Clear();
  translation.Clear();

  try
    reReport.lines.add(_('Loading original...'));
    original.LoadFromFile(feOriginal.Filename);
    JoinLines(original);
  except
    on E:Exception do
      begin
      reReport.lines.add(Format(_('Unable to load: %s'), [feOriginal.Filename]));
      allOK := FALSE;
      end;
  end;

  try
    reReport.lines.add(_('Loading translation...'));
    translation.LoadFromFile(feTranslated.Filename);
    JoinLines(translation);
  except
    on E:Exception do
      begin
      reReport.lines.add(Format(_('Unable to load: %s'), [feTranslated.Filename]));
      allOK := FALSE;
      end;
  end;

  Result := allOK;
end;

procedure TMain.pbCheckClick(Sender: TObject);
var
  original: TStringList;
  translation: TStringList;
  tIdx: integer;
  i: integer;
  strDefault: string;
  strTranslated: string;
  problemCnt: integer;
begin
  reReport.lines.clear();
  
  original:= TStringList.Create();
  translation:= TStringList.Create();
  try
    if LoadTranslation(original, translation) then
      begin
      problemCnt := 0;
      reReport.lines.add(_('Checking translation...'));
      reReport.lines.add('');

      for i:=0 to (original.count - 1) do
        begin
        if (original[i] = 'msgid ""') then
          begin
          continue;
          end;

        if (Pos('msgid "', original[i]) > 0) then
          begin
          tIdx := translation.IndexOf(original[i]);
          if (tIdx < 0) then
            begin
            inc(problemCnt);
            reReport.lines.add(Format(_('No translation found for: %s'), [original[i]]));
            end
          else
            begin
            strDefault := original[i];
            strTranslated := translation[tIdx+1];

            strDefault := Copy(strDefault, length('msgid ')+1, (length(strDefault) - length('msgid ')));
            strTranslated := Copy(strTranslated, length('msgstr ')+1, (length(strTranslated) - length('msgstr ')));
            if not(CheckParameters(strDefault, strTranslated)) then
              begin
              inc(problemCnt);
              reReport.lines.add(Format(_('Parameters different for: %s'), [original[i]]));
              end;
            end;
          end;
        end;

      reReport.lines.add('');
      reReport.lines.add(_('Finished.'));
      if (problemCnt = 0) then
        begin
        reReport.lines.add(_('Check found no problems'));
        end
      else
        begin
        reReport.lines.add(Format(_('Found %d issue(s)'), [problemCnt]));
        end;
        
      end;

  finally
    translation.Free();
    original.Free();
  end;

end;

// Strip message down to just it's "%" parameters
// NOTICE: This only strips to the "%" and the *single* character following it
function TMain.StripToPlaceholderes(s: string): string;
var
  retval: string;
  i: integer;
begin
  retval := '';

  i := 1;
  while (i <= length(s)) do
    begin
    if (s[i] = '%') then
      begin
      retval := retval + s[i] + s[i+1];
      inc(i);
      end;

    inc(i);
    end;

  Result := retval;
end;

// %1, %2...%n style parameters - only check that valid parameters appear in
// the translation, and that all parameters do appear
function TMain.CheckParameters_123Style(strippedOriginal: string; strippedTranslation: string): boolean;
var
  orig: TStringList;
  trans: TStringList;
  i: integer;
  retval: boolean;
begin
  retval := TRUE;

  orig:= TStringList.Create();
  trans:= TStringList.Create();
  try
    orig.DelimitedText := StringReplace(strippedOriginal, '%', orig.Delimiter, [rfReplaceAll]);
    trans.DelimitedText := StringReplace(strippedTranslation, '%', trans.Delimiter, [rfReplaceAll]);

    // Check that all of the %n parameters in the translated version appear in
    // the original version
    // i.e. That the translation doesn't add additional parameters
    for i:=0 to (trans.count-1) do
      begin
      // Ignore any initial blank
      if (trans[i] = '') then
        begin
        continue;
        end;

      if (orig.IndexOf(trans[i]) < 0) then
        begin
        retval := FALSE;
        break;
        end;
      end;

    // Check that all of the %n parameters in the original version appear in
    // the translated version
    // i.e. That the translation doesn't miss out any parameters
    for i:=0 to (orig.count-1) do
      begin
      // Ignore any initial blank
      if (orig[i] = '') then
        begin
        continue;
        end;

      if (trans.IndexOf(orig[i]) < 0) then
        begin
        retval := FALSE;
        break;
        end;
      end;

  finally
    orig.Free();
    trans.Free();
  end;

  Result := retval;
end;

// Check that "%..." parameters are all present, and in equal numbers
function TMain.CheckParameters(original: string; translation: string): boolean;
var
  retval: boolean;
  stripOriginal: string;
  stripTranslation: string;
begin
  retval := TRUE;
  if (
      (original = '""') or
      (translation = '""')
     ) then
    begin
    // Do nothing - retval already TRUE
    end
  else
    begin
    stripOriginal:= StripToPlaceholderes(original);
    stripTranslation:= StripToPlaceholderes(translation);

    if (rgParamCheck.ItemIndex = 0) then
      begin
//    reReport.lines.add(_('Parameter checks: Skipped'));
      end
    else if (rgParamCheck.ItemIndex = 1) then
      begin
//    reReport.lines.add(_('Parameter checks: %1, %2...%n style'));
      retval := CheckParameters_123Style(stripOriginal, stripTranslation);
      end
    else if (rgParamCheck.ItemIndex = 2) then
      begin
//    reReport.lines.add(_('Parameter checks: %s, %d style'));
      // Position significant, parameters must match *exactly*
      retval := (stripOriginal = stripTranslation);
      end;

    end;

  Result := retval;
end;

procedure TMain.pbCloseClick(Sender: TObject);
begin
  Close();
end;

procedure TMain.FormResize(Sender: TObject);
begin
  SDUCenterControl(pbCheck, ccHorizontal);
end;

procedure TMain.FormShow(Sender: TObject);
begin
  self.Caption := Application.Title;
  reReport.lines.Clear();

  feOriginal.Filter     := 'Translation files (*.po)|*.po|All files|*.*';
  feOriginal.DefaultExt := '.po';
  feTranslated.Filter     := 'Translation files (*.po)|*.po|All files|*.*';
  feTranslated.DefaultExt := '.po';

end;

END.

